/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.net.impl;

import org.xmloperator.lambda.net.exception.UnexpectedAnnihilationException;
import org.xmloperator.lambda.net.model.Applicator;
import org.xmloperator.lambda.net.model.ConnectionListener;
import org.xmloperator.lambda.net.model.Operator;
import org.xmloperator.lambda.net.model.Port;

/**
 * Applicator implementation.
 */
final class ApplicatorImpl extends OperatorImpl implements Applicator {
	final PortImpl input;
	final PortImpl body;
	final PortImpl argument;
	boolean hasOrientationReversed = false;

	ApplicatorImpl(OperatorFactoryImpl operatorFactory, int mark) {
		super(operatorFactory, mark);
		this.body = new PortImpl(Port.TYPE_APPLICATOR_BODY, this, 0, false);
		this.input = new PortImpl(Port.TYPE_APPLICATOR_INPUT, this, 1, true);
		this.argument = new PortImpl(Port.TYPE_APPLICATOR_ARGUMENT, this, 2, false);
	}

	//////////////////////////
	// Operator implementation
	//////////////////////////

	public boolean canBeReturnedToFactory() {
		return super.canBeReturnedToFactory()
			&& this.input.connectedPort == null
			&& this.body.connectedPort == null
			&& this.argument.connectedPort == null;
	}

	public void returnToFactory() {
		super.returnToFactory();
		this.hasOrientationReversed = false;
		this.body.number = 0;
		this.input.number = 1;
		this.operatorFactory.returnApplicator(this);
	}

	public int getPortCount() {
		return 3;
	}

	public int getOperatorType() {
		return Operator.TYPE_APPLICATOR;
	}

	public boolean isApplicator() {
		return true;
	}

	public Port getPort(int number) {
		switch (number) {
			case 0 :
				return this.hasOrientationReversed
					? (Port) this.input
					: (Port) this.body;
			case 1 :
				return this.hasOrientationReversed
					? (Port) this.body
					: (Port) this.input;
			case 2 :
				return this.argument;
			default :
				throw new IllegalArgumentException();
		}
	}

	public Port getPort0() {
		return this.hasOrientationReversed ? (Port) this.input : (Port) this.body;
	}

	public Port getPort1() {
		return this.hasOrientationReversed ? (Port) this.body : (Port) this.input;
	}

	public Port getPort2() {
		return this.argument;
	}

	public boolean isDirectionInversed() {
		return false;
	}

	public Operator cloneOperator(boolean isDirectionInversed) {
		if (isDirectionInversed)
			throw new IllegalArgumentException();
		Applicator applicator = this.operatorFactory.createApplicator(this.mark);
		if (this.hasOrientationReversed)
			applicator.reverseOrientation();
		return applicator;
	}

	public int interact(ConnectionListener xListener) {
		Operator connectedOperator =
			this.getPort0().getConnectedPort().getOperator();
		if (connectedOperator.isAbstractor())
			return Beta.interact(this, (AbstractorImpl) connectedOperator, xListener);
		if (connectedOperator.isApplicator())
			throw new UnexpectedAnnihilationException(this);
		return connectedOperator.interact(xListener);
	}

	////////////////////////////
	// Applicator implementation
	////////////////////////////

	public Port getInput() {
		return this.input;
	}

	public Port getBody() {
		return this.body;
	}

	public Port getArgument() {
		return this.argument;
	}

	public void reverseOrientation() {
		if (this.hasOrientationReversed)
			throw new IllegalStateException();
		this.hasOrientationReversed = true;
		this.input.number = 0;
		this.body.number = 1;
	}
}
