/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.net.impl;

import org.xmloperator.lambda.net.exception.MarkedDifferentlyException;
import org.xmloperator.lambda.net.model.ConnectionListener;
import org.xmloperator.lambda.net.model.Delimiter;
import org.xmloperator.lambda.net.model.Operator;
import org.xmloperator.lambda.net.model.OperatorFactory;
import org.xmloperator.lambda.net.model.Port;

/**
 * Beta interaction.
 */
abstract class Beta {

	/**
	 * The Applicator "A" interacts with the Abstractor "L" :
	 * (B--A>--<L--C) -> (B--<D--C)
	 */
	static final int interact(
		ApplicatorImpl applicator,
		AbstractorImpl abstractor,
		ConnectionListener xListener) {
		Port portA0 = applicator.getPort0();
		if (portA0.getConnectedPort() != abstractor.getPort0())
			throw new IllegalArgumentException();
		if (applicator.hasOrientationReversed)
			throw new IllegalStateException();
		int mark = applicator.mark;
		if (abstractor.mark != mark)
			throw new MarkedDifferentlyException(abstractor);
		Port portA1 = applicator.input;
		Port portA2 = applicator.argument;
		Port portB1 = portA1.getConnectedPort();
		Port portB2 = portA2.getConnectedPort();
		if (portB1 == portA2 || portB2 == portA1)
			throw new IllegalStateException();
		Port portL1 = abstractor.body;
		Port portL2 = abstractor.bind;
		Port portC1 = portL1.getConnectedPort();
		Port portC2 = portL2.getConnectedPort();
		portA0.disconnect();
		portA1.disconnect();
		portA2.disconnect();
		portL1.disconnect();
		portL2.disconnect();
		OperatorFactory operatorFactory = applicator.getFactory();
		abstractor.returnToFactory();
		applicator.returnToFactory();
		if (portC1 == portL2) {
			if (portC2 != portL1)
				throw new IllegalStateException();
			portB1.connectTo(portB2);
			xListener.notifyConnection(portB1, portB2);
			return Operator.INTERACTION_BETA_IDENTITY;
		}
		{
			Delimiter delimiterD1 = operatorFactory.createDelimiter(mark, true, null);
			Port portD10 = delimiterD1.getPort0();
			Port portD11 = delimiterD1.getPort1();
			portB1.connectTo(portD10);
			xListener.notifyConnection(portB1, portD10);
			portD11.connectTo(portC1); // Not a principal connection.
		}
		Operator operatorC2 = portC2.getOperator();
		if (operatorC2.isEraser()) {
			operatorC2.returnToFactory();
			Pruner.returnBranchToFactory(portB2, true);
		}
		else {
			Delimiter delimiterD2 =
				operatorFactory.createDelimiter(mark, false, null);
			Port portD20 = delimiterD2.getPort0();
			Port portD21 = delimiterD2.getPort1();
			portB2.connectTo(portD20);
			xListener.notifyConnection(portB2, portD20);
			portD21.connectTo(portC2); // Not a principal connection.
		}
		return Operator.INTERACTION_BETA_REDUCTION;
	}
}
