/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.net.impl;

import org.xmloperator.lambda.net.model.Operator;
import org.xmloperator.lambda.net.model.OperatorFactory;

/**
 * Operator implementation.
 */
abstract class OperatorImpl implements Operator {
	final OperatorFactoryImpl operatorFactory;
	boolean isReturnedToTheFactory = false;
	int mark;

	OperatorImpl(OperatorFactoryImpl operatorFactory, int mark) {
		this.operatorFactory = operatorFactory;
		this.mark = mark;
	}

	//////////////////////////
	// Operator implementation
	//////////////////////////

	public final OperatorFactory getFactory() {
		return this.operatorFactory;
	}

	public final boolean isReturnedToFactory() {
		return this.isReturnedToTheFactory;
	}

	public boolean canBeReturnedToFactory() {
		return !this.isReturnedToTheFactory;
	}

	public void returnToFactory() {
		if (!canBeReturnedToFactory())
			throw new IllegalStateException();
		this.mark = 0;
		this.isReturnedToTheFactory = true;
	}

	public boolean isEraser() {
		return false;
	}

	public boolean isDelimiter() {
		return false;
	}

	public boolean isAbstractor() {
		return false;
	}

	public boolean isApplicator() {
		return false;
	}

	public boolean isDuplicator() {
		return false;
	}

	public final int getMark() {
		return this.mark;
	}

	public final void setMark(int mark) {
		this.mark = mark;
	}
}
