/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.net.model;

/**
 * Applicator in a lambda-net.
 */
public interface Applicator extends Operator {

	/**
	 * Returns its input Port. It is the principal Port if the orientation is
	 *   reversed, the first auxiliary Port if the orientation is normal.
	 * @return its input Port.
	 */
	public Port getInput();
	
	/**
	 * Returns its body Port. It is the principal Port if the orientation is
	 *   normal, the first auxiliary Port if the orientation is reversed.
	 *   This is an output Port.
	 * @return its body Port.
	 */
	public Port getBody();

	/**
	 * Returns its argument Port. This is the second auxiliary Port.
	 *   This is an output Port.
	 * @return its argument Port.
	 */
	public Port getArgument();

	/**
	 * Reverses the orientation of the principal Port.
	 * Pre-condition: the principal Port is the body Port (normal orientation).
	 * Post-condition: the principal Port is the input Port (reversed orientation).
	 * @throws IllegalStateException if the orientation is already reversed.
	 */
	public void reverseOrientation();
}
