/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.net.model;

/**
 * Port of an Operator in a lambda-net.
 * @see org.xmloperator.lambda.net.model.Operator
 */
public interface Port {
	public static final int PORT_TYPE_COUNT = 12;
	public static final int TYPE_ERASER_PORT = 0;
	public static final int TYPE_DELIMITER_INPUT = 1;
	public static final int TYPE_DELIMITER_BODY = 2;
	public static final int TYPE_ABSTRACTOR_INPUT = 3;
	public static final int TYPE_ABSTRACTOR_BODY = 4;
	public static final int TYPE_ABSTRACTOR_BIND = 5;
	public static final int TYPE_APPLICATOR_INPUT = 6;
	public static final int TYPE_APPLICATOR_BODY = 7;
	public static final int TYPE_APPLICATOR_ARGUMENT = 8;
	public static final int TYPE_DUPLICATOR_INPUT1 = 9;
	public static final int TYPE_DUPLICATOR_INPUT2 = 10;
	public static final int TYPE_DUPLICATOR_OUTPUT = 11;
	
	/**
	 * Returns the type of this Port.
	 * @return the type of this Port. It is one of the constants TYPE_*.
	 */
	public int getPortType();
	
	/**
	 * Returns the Operator to which this Port belongs.
	 * @return the Operator to which this Port belongs.
	 */
	public Operator getOperator();
	
	/**
	 * Returns the number of this Port. It may depend on the Operator orientation.
	 * @return the number of this Port. This one of the Operator.NUMBER_*
	 *   constants.
	 */
	public int getNumber();
	
	/**
	 * Returns <tt>true</tt> if this Port is the principal one of its Operator.
	 * @return <tt>true</tt> if this Port is the principal one of its Operator.
	 */
	public boolean isPrincipal();

	/**
	 * Returns <tt>true</tt> if this Port is the first auxiliary Port of its
	 *   Operator.
	 * @return <tt>true</tt> if this Port is the first auxiliary Port of its
	 *   Operator.
	 */
	public boolean isFirstAuxiliary();
	
	/**
	 * Returns <tt>true</tt> if this Port is the second auxiliary Port of its
	 *   Operator.
	 * @return <tt>true</tt> if this Port is the second auxiliary Port of its
	 *   Operator.
	 */
	public boolean isSecondAuxiliary();
	
	/**
	 * Returns <tt>true</tt> if this Port is an input Port, false if it is an
	 *   output Port.
	 * @return <tt>true</tt> if this Port is an input Port, false if it is an
	 *   output Port.
	 */
	public boolean isInput();
		
	/**
	 * Returns the connected Port if any, null elsewhere.
	 * @return the connected Port if any, null elsewhere.
	 */
	public Port getConnectedPort();
	
	/**
	 * Connect this Port to a another Port.
	 * If connected, disconnecting is required before connecting to another Port.
	 * @param connectedPort a Port to connect.
	 * @throws org.xmloperator.lambda.net.exception.AlreadyConnectedException
	 *   on trying to connect two Ports whose one or the two ones are already
	 *   connected.
	 * @throws IllegalArgumentException if the given Port is null or itself.
	 * @throws org.xmloperator.lambda.net.exception.SameTypePortException if the
	 *   given Port has the same type (input or output) than this Port.
	 */
	public void connectTo(Port connectedPort);

	/**
	 * Disconnect this Port.
	 */
	public void disconnect();
}
