/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.net.reduction;

import org.xmloperator.lambda.net.model.Applicator;
import org.xmloperator.lambda.net.model.Eraser;

/**
 * Beta reduction counter.
 * May be used to stop iterations when reaching a max count.
 */
public class ReductionCounter implements ReductionListener {
	public static final int UNLIMITED = -1;

	public ReductionListener nextReductionListener = null;
	private final int maxCount;
	private int count = 0;

	public ReductionCounter(int maxCount) {
		this.maxCount = maxCount;
	}

	public int getCount() {
		return this.count;
	}

	public void clear() {
		this.count = 0;
	}

	///////////////////////////////
	// RedexListener implementation
	///////////////////////////////

	public void notifyStartReduction(
		Eraser rootEraser,
		Applicator redexApplicator) throws NotInNormalFormException {
		if (this.count == this.maxCount)
			throw new NotInNormalFormException();
		ReductionListener reductionListener = this.nextReductionListener;
		if (reductionListener != null)
			reductionListener.notifyStartReduction(rootEraser, redexApplicator);
		this.count++;
	}
}
