/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.net.util;

import java.util.HashMap;
import java.util.Map;

import org.xmloperator.lambda.net.exception.FreePortException;
import org.xmloperator.lambda.net.model.Eraser;
import org.xmloperator.lambda.net.model.Operator;
import org.xmloperator.lambda.net.model.OperatorFactory;
import org.xmloperator.lambda.net.model.Port;
import org.xmloperator.lambda.net.model.PortStack;

/**
 * Utilities about cloning a lambda-net.
 */
public abstract class CloneUtils {
	private static final boolean ASSERTION = Test.ASSERTION;

	/**
	 * Clones a lambda-net.
	 * @param rootEraser a root Eraser.
	 * @return the clone root Eraser.
	 * @throws FreePortException if the lambda-net is not closed.
	 */
	public static final Eraser cloneNet(final Eraser rootEraser) {
		if (ASSERTION)
			NetUtils.verifyClosure(rootEraser);
		final Eraser cloneRootEraser =
			(Eraser) rootEraser.cloneOperator(rootEraser.isDirectionInversed());
		final Map map = new HashMap();
		final OperatorFactory operatorFactory = rootEraser.getFactory();
		final PortStack portStack = operatorFactory.createPortStack();
		try {
			portStack.push(rootEraser.getPort());
			portStack.push(cloneRootEraser.getPort());
			while (!portStack.isEmpty()) {
				Port clonePort = portStack.pop();
				Port port = portStack.pop();
				if (clonePort.getConnectedPort() == null) {
					Port connectedPort = port.getConnectedPort();
					if (connectedPort == null)
						throw new FreePortException(port);
					int portNumber = connectedPort.getNumber();
					Operator operator = connectedPort.getOperator();
					Operator cloneOperator = (Operator) map.get(operator);
					if (cloneOperator != null)
						clonePort.connectTo(cloneOperator.getPort(portNumber));
					else {
						cloneOperator =
							operator.cloneOperator(operator.isDirectionInversed());
						map.put(operator, cloneOperator);
						clonePort.connectTo(cloneOperator.getPort(portNumber));
						switch (operator.getPortCount()) {
							case 3 :
								if (!connectedPort.isSecondAuxiliary()) {
									portStack.push(operator.getPort2());
									portStack.push(cloneOperator.getPort2());
								}
							case 2 :
								if (!connectedPort.isFirstAuxiliary()) {
									portStack.push(operator.getPort1());
									portStack.push(cloneOperator.getPort1());
								}
								if (!connectedPort.isPrincipal()) {
									portStack.push(operator.getPort0());
									portStack.push(cloneOperator.getPort0());
								}
						}
					}
				}
			}
			if (ASSERTION)
				NetUtils.verifyClosure(cloneRootEraser);
			return cloneRootEraser;
		}
		finally {
			portStack.returnToFactory();
		}
	}
}
