/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.net.util;

import org.xmloperator.lambda.net.model.Operator;

/**
 * Interaction statistic.
 */
public class InteractionStat implements InteractionListener {
	private static final int CLASS_COUNT = 6;
	private static final int CLASS_BETA = 0;
	private static final int CLASS_EXTRUDE = 1;
	private static final int CLASS_ANNIHILATE = 2;
	private static final int CLASS_COMMUTE = 3;
	private static final int CLASS_DUPLICATE = 4;
	private static final int CLASS_ERASE = 5;
	private static final String[] CLASS_NAME =
		{ "Beta", "Extrude", "Annihilate", "Commute", "Duplicate", "Erase" };
	private static final int[] FROM_TYPE_TO_CLASS =
		{
			CLASS_BETA,
			CLASS_BETA,
			CLASS_EXTRUDE,
			CLASS_EXTRUDE,
			CLASS_EXTRUDE,
			CLASS_EXTRUDE,
			CLASS_EXTRUDE,
			CLASS_ANNIHILATE,
			CLASS_ANNIHILATE,
			CLASS_ANNIHILATE,
			CLASS_COMMUTE,
			CLASS_COMMUTE,
			CLASS_DUPLICATE,
			CLASS_DUPLICATE,
			CLASS_DUPLICATE,
			CLASS_ERASE,
			CLASS_ERASE,
			CLASS_ERASE,
			CLASS_ERASE,
			CLASS_ERASE };

	static {
		if (FROM_TYPE_TO_CLASS.length != Operator.INTERACTION_TYPE_COUNT)
			throw new IllegalStateException();
	}

	private final int[] typeCounts = new int[Operator.INTERACTION_TYPE_COUNT];

	public InteractionStat() {
		clear();
	}

	public void clear() {
		for (int type = 0; type < this.typeCounts.length; type++)
			this.typeCounts[type] = 0;
	}

	public int betaCount() {
		return this.typeCounts[Operator.INTERACTION_BETA_REDUCTION]
			+ this.typeCounts[Operator.INTERACTION_BETA_IDENTITY];
	}

	public int classCount(int interactionClass) {
		int count = 0;
		for (int k = 0; k < this.typeCounts.length; k++)
			if (FROM_TYPE_TO_CLASS[k] == interactionClass)
				count += this.typeCounts[k];
		return count;
	}

	public int totalCount() {
		int count = 0;
		for (int k = 0; k < this.typeCounts.length; k++)
			count += this.typeCounts[k];
		return count;
	}

	public static final int REPORT_STYLE_DETAIL = 1;
	public static final int REPORT_STYLE_CLASS = 2;
	public static final int REPORT_STYLE_SYNTHETIC = 3;

	public String report(int reportStyle) {
		StringBuffer sb = new StringBuffer();
		sb.append('[');
		switch (reportStyle) {
			case REPORT_STYLE_DETAIL :
				for (int k = 0; k < this.typeCounts.length; k++) {
					int count = this.typeCounts[k];
					if (k != 0)
						sb.append(", ");
					sb.append(k);
					sb.append('(');
					sb.append(count);
					sb.append(')');
				}
				break;
			case REPORT_STYLE_CLASS :
				for (int k = 0; k < CLASS_COUNT; k++) {
					if (k != 0)
						sb.append(", ");
					sb.append(CLASS_NAME[k]);
					sb.append('(');
					sb.append(this.classCount(k));
					sb.append(')');
				}
				break;
			case REPORT_STYLE_SYNTHETIC :
				sb.append(this.betaCount());
				sb.append('/');
				sb.append(this.totalCount());
				break;
		}
		sb.append(']');
		return sb.toString();
	}

	public String toString() {
		return report(REPORT_STYLE_SYNTHETIC);
	}

	/////////////////////////////////////
	// InteractionListener implementation
	/////////////////////////////////////

	public void notifyInteraction(int interactionReport) {
		int interactionType = interactionReport & 0xFF;
		this.typeCounts[interactionType]++;
	}
}
