/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.net.util;

import java.util.HashSet;
import java.util.Set;

import org.xmloperator.lambda.net.model.Abstractor;
import org.xmloperator.lambda.net.model.Eraser;
import org.xmloperator.lambda.net.model.Operator;
import org.xmloperator.lambda.net.model.ScopeBinder;

/**
 * Counter of ScopeBinders, Abstractors and Delimiters.
 */
public final class ScopeBinderCounter {
	public int scopeBinderCount = 0;
	public int boundAbstractorCount = 0;
	public int boundDelimiterCount = 0;
	public int unboundAbstractorCount = 0;

	public ScopeBinderCounter() {
	}

	/**
	 * Counts in a lambda-net the number of :
	 * - ScopeBinder,
	 * - Abstractor bound to a ScopeBinder,
	 * - Delimiter bound to a ScopeBinder,
	 * - unbound Abstractor (they are closed Abstractors).
	 * @param rootEraser a root <tt>Eraset</tt>
	 */
	public final void count(Eraser rootEraser) {
		int boundAbstractorCount = 0;
		int boundDelimiterCount = 0;
		int unboundAbstractorCount = 0;
		Set scopeBinderSet = new HashSet();
		OperatorIterator iterator = new NetTraversal(rootEraser, true);
		iterator.next(); // Root Eraser.
		while (iterator.hasNext()) {
			Operator operator = iterator.next();
			if (operator.isAbstractor()) {
				Abstractor abstractor = (Abstractor) operator;
				ScopeBinder scopeBinder = abstractor.getScopeBinder();
				if (scopeBinder != null) {
					if (scopeBinderSet.add(scopeBinder)) {
						boundAbstractorCount += scopeBinder.getAbstractorList().getSize();
						boundDelimiterCount += scopeBinder.getDelimiterList().getSize();
					}
				}
				else
					unboundAbstractorCount++;
			}
		}
		NetUtils.incrementMark(rootEraser, false);
		this.scopeBinderCount = scopeBinderSet.size();
		this.boundAbstractorCount = boundAbstractorCount;
		this.boundDelimiterCount = boundDelimiterCount;
		this.unboundAbstractorCount = unboundAbstractorCount;
	}

	public String toString() {
		return scopeBinderCount
			+ " binder("
			+ boundAbstractorCount
			+ " abs., "
			+ boundDelimiterCount
			+ " del.), "
			+ unboundAbstractorCount
			+ " free abs.";
	}
}
