/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.shared;

import java.util.Arrays;

/**
 * ObjectArrayFactory Implementation.
 */
public class ObjectArrayFactoryImpl implements ObjectArrayFactory {

	/**
	 * Set to true, the factory reuses returned objects.
	 * Set to false, returned objects are garbaged.
	 */
	private static final boolean REUSE = true;

	private static final int POWER_MIN = 3;
	private static final int POWER_MAX = 30;
	private static final int STACK_STACK_LENGTH = POWER_MAX - POWER_MIN + 1;
	private final Stack[] stackStack = new Stack[STACK_STACK_LENGTH];
	private final int[] stackLength = new int[STACK_STACK_LENGTH];
	private final int[] newStackCount = new int[STACK_STACK_LENGTH];

	public ObjectArrayFactoryImpl() {
		for (int k = 0; k < this.stackStack.length; k++) {
			this.stackStack[k] = new Stack();
			this.stackLength[k] = 1 << (POWER_MIN + k);
		}
	}

	/**
	 * Clear memory.
	 */
	public void clear() {
		for (int k = 0; k < STACK_STACK_LENGTH; k++) {
			this.stackStack[k].clear();
			this.newStackCount[k] = 0;
		}
	}

	/**
	 * @return true if all the objects that have been created have been returned.
	 */
	public boolean isWellBalanced() {
		for (int k = 0; k < STACK_STACK_LENGTH; k++)
			if (this.stackStack[k].getSize() != this.newStackCount[k])
				return false;
		return true;
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append('[');
		for (int k = 0; k < STACK_STACK_LENGTH; k++) {
			if (k != 0)
				sb.append(", ");
			sb.append(POWER_MIN + k);
			sb.append('(');
			sb.append(this.newStackCount[k]);
			sb.append(',');
			sb.append(this.stackStack[k].getSize());
			sb.append(')');
		}
		sb.append(']');
		return sb.toString();
	}

	////////////////////////////////////
	// ObjectArrayFactory implementation
	////////////////////////////////////

	public Object[] createObjectArray(int length) {
		if (length <= 0 || ((length - 1) & length) != 0)
			throw new IllegalArgumentException();
		int index = 0;
		if (length > this.stackLength[0])
			for (int k = 1; k < STACK_STACK_LENGTH; k++)
				if (length == this.stackLength[k]) {
					index = k;
					break;
				}
		if (REUSE) {
			Object[] objectArray = (Object[]) this.stackStack[index].pop();
			if (objectArray != null)
				return objectArray;
		}
		this.newStackCount[index]++;
		return new Object[this.stackLength[index]];
	}

	public void returnObjectArray(Object[] objectArray) {
		int length = objectArray.length;
		int index = -1;
		for (int k = 0; k < STACK_STACK_LENGTH; k++)
			if (length == this.stackLength[k]) {
				index = k;
				break;
			}
		if (index < 0)
			throw new IllegalArgumentException();
		Arrays.fill(objectArray, null);
		if (REUSE)
			this.stackStack[index].push(objectArray);
	}
}
