/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.tree.impl;

import org.xmloperator.lambda.tree.exception.ReturnedToFactoryException;
import org.xmloperator.lambda.tree.model.Application;
import org.xmloperator.lambda.tree.model.Term;

/**
 * Application implementation.
 */
final class ApplicationImpl extends TermWithBodyImpl implements Application {
	private TermImpl argument;
	
	ApplicationImpl(TermFactoryImpl termFactory) {
		super(termFactory);
	}

	//////////////////////
	// Term implementation
	//////////////////////

	public boolean canBeReturnedToFactory() {
		return super.canBeReturnedToFactory() && this.argument == null;
	}

	public void returnToFactory() {
		super.returnToFactory();
		this.termFactory.returnApplication(this);
	}

	public int getType() {
		return Term.TYPE_APPLICATION;
	}
	
	public boolean isApplication() {
		return true;
	}
	
	/////////////////////////////
	// Application implementation
	/////////////////////////////
	
	public Term getArgument() {
		return this.argument;
	}

	public void setArgument(Term argument) {
		if (this.isReturnedToTheFactory)
			throw new ReturnedToFactoryException(this);
		if (argument == null || argument.getParent() != null)
			throw new IllegalArgumentException();
		TermImpl oldArgument = this.argument;
		TermImpl newArgument = (TermImpl)argument;
		if (oldArgument != null)
			oldArgument.parent = null;
		newArgument.parent = this;
		this.argument = newArgument;
	}

	public void clearArgument() {
		TermImpl oldArgument = this.argument;
		if (oldArgument != null)
			oldArgument.parent = null;
		this.argument = null;
	}
}
