/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.tree.impl;

import org.xmloperator.lambda.tree.exception.ReturnedToFactoryException;
import org.xmloperator.lambda.tree.model.Term;
import org.xmloperator.lambda.tree.model.TermWithBody;

/**
 * TermWithBody implementation.
 */
abstract class TermWithBodyImpl extends TermImpl implements TermWithBody {
	private TermImpl body = null;
	
	TermWithBodyImpl(TermFactoryImpl termFactory) {
		super(termFactory);
	}
	
	//////////////////////
	// Term implementation
	//////////////////////
	
	public boolean canBeReturnedToFactory() {
		return super.canBeReturnedToFactory() && this.body == null;
	}

	//////////////////////////////
	// TermWithBody implementation
	//////////////////////////////
	
	public Term getBody() {
		return this.body;
	}

	public void setBody(Term body) {
		if (this.isReturnedToTheFactory)
			throw new ReturnedToFactoryException(this);
		if (body == null || body.getParent() != null)
			throw new IllegalArgumentException();
		TermImpl oldBody = this.body;
		TermImpl newBody = (TermImpl)body;
		if (oldBody != null)
			oldBody.parent = null;
		newBody.parent = this;
		this.body = newBody;
	}

	public void clearBody() {
		TermImpl oldBody = this.body;
		if (oldBody != null)
			oldBody.parent = null;
		this.body = null;
	}
}
