/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.tree.model;

/**
 * Term with body.
 * This is either an <tt>EndOfScope</tt>, an <tt>Abstraction</tt> or an
 *   <tt>Application</tt>.
 */
public interface TermWithBody extends Term {

	/**
	 * Returns the body of this Term.
	 * The body of an Application is its function.
	 * @return the body of this Term.
	 */
	public Term getBody();
	
	/**
	 * Set the body of this Term.
	 * The body of an Application is its function.
	 * @param body a Term.
	 * @throws IllegalArgumentException if the given Term is null or has
	 *   already a parent.
	 * @throws IllegalCastException if the implementation of the body
	 *   is not compatible with the implementation of this Term.
	 * @throws ReturnedToTheFactoryException if this Term has been returned to
	 *   the factory.
	 */
	public void setBody(Term body);

	/**
	 * Clear the body of this Term.
	 */
	public void clearBody();
}
