/**
 * Copyright (c) 2004 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/lambda/).
 */
package org.xmloperator.lambda.tree.util;

import org.xmloperator.lambda.tree.model.Abstraction;
import org.xmloperator.lambda.tree.model.Application;
import org.xmloperator.lambda.tree.model.EndOfScope;
import org.xmloperator.lambda.tree.model.Term;
import org.xmloperator.lambda.tree.model.TermFactory;
import org.xmloperator.lambda.tree.model.TermStack;

/**
 * Utilities about signature of Terms.
 */
public abstract class SignatureUtils {

	/**
	 * Returns the count of Leaf and Abstraction within a Term.
	 * This may be used as a size measurement of closed Terms since there is a
	 *   finite number of closed Term having such a given size.
	 * @param rootTerm a Term.
	 * @return the count of Leaf and Abstraction within the given Term.
	 * @throws NullPointerException if the given Term is broken.
	 */
	public static final int leafAndAbstractionCount(final Term rootTerm) {
		TermFactory termFactory = rootTerm.getFactory();
		TermStack termStack = termFactory.createTermStack(false);
		try {
			int count = 0;
			Term term = rootTerm;
			while (true) {
				if (term.isLeaf())
					count++;
				else if (term.isEndOfScope()) {
					EndOfScope endOfScope = (EndOfScope) term;
					term = endOfScope.getBody();
					continue;
				}
				else if (term.isAbstraction()) {
					Abstraction abstraction = (Abstraction) term;
					count++;
					term = abstraction.getBody();
					continue;
				}
				else if (term.isApplication()) {
					Application application = (Application) term;
					termStack.push(application.getArgument());
					term = application.getBody();
					continue;
				}
				else
					throw new IllegalArgumentException();
				if (termStack.isEmpty())
					break;
				term = termStack.pop();
			}
			return count;
		}
		finally {
			termStack.returnToFactory();
		}
	}
}
